% This code is a adaptation of Koop and Potter (2007) 
% "Estimation and Forecasting in Models with Multiple Breaks, Review of Economic Studies
% to the selection of the number clusters of turning points
%"A new approach to dating the reference cycle"
% by Maximo Camacho, Lola Gadea, and Ana Gomez-Loscos (2018)
% Journal of Business % Economic Statistics
% May 2020
clear
clc

rng(1,'twister');
seed=rng;
warning ('off','all');
tic
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      Please, adjust the paths                           %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
p1 = genpath('X:\...\functions_empirical');
path(path,p1) %general auxiliary matlab functions
p2 = genpath('X:\...\general_functions');
path(path,p2) %specific matlab functions of the empirical section

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    Read data and manipulate the data                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[data]=xlsread('data_nber.xlsx','NBER','A1:K622');
X=data;
dates=data(1:end-1,1);
X=X(1:size(X,1)-1,2:end);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                      CALL BRY-BOSCHAN                                   %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[t, n]=size(X);
r=2; %bivariate normals
STATES=NaN(t,n);
PEAKS=NaN(t,n);
TROUGHS=NaN(t,n);
for i=1:n
    clc;
    fprintf('Bry-Boschan to series %4.0f of %4.0f\n',i,n);
    x=X(:,i);
    [l1, l2]=position_nan(x);
    x= x(~isnan(x));
    l=length(x);
    [states,peaks,troughs,index_peaks, index_troughs] = bryboschan_me(x,1);
    STATES(l1:l2,i)=states;
    PEAKS(l1:l2,i)=index_peaks;
    TROUGHS(l1:l2,i)=index_troughs;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                 Building bi-variate data                                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Build pairs of peaks/troughs or troughs/peaks from BB index of peaks and trougs
% Start returns P if the economy starts by a peak and T if starts by a trough
[PAIR, start]=preparing_file_bi(PEAKS,TROUGHS,dates); 
if start==1
    fprintf('THE ECONOMY STATRS BY A PEAK\n')
elseif start==2
    fprintf('THE ECONOMY STATRS BY A TROUGH\n')
end
Y=PAIR;
t=size(Y,1);
save('Y.dat','Y','-ASCII')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Initial value mu1_0 by K-modes                %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
MU_MAT=Y;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    Prior hyperparmaters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lambda=ones(t,1)*12;

%alpha is a in G(a,b) for Poisson intensity
alpha=5;

%for beta(lambda)^-1 we have prior G(hyp_b1,1/hyp_b2)
hyp_b1=alpha;
hyp_b2=1/hyp_b1;
beta=gamrnd(hyp_b1,hyp_b2,1,1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    Storage results
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
s=zeros(t,1);
maxb=zeros(t+1,1);  % max number of extended regimes assuming start with duration 1 in first regime
maxb(1,:)=1;
for i=2:t+1
    maxb(i,:)=maxb(i-1,:)+(i-1)*i/2+1;
end

slnf=lnfact(seqa(0,1,t));  %Logarithm of the factorial of a trend
lgib=100;  %burn in phase
totgib=1000;

K_=zeros(totgib,1);
LAMBDA_=zeros(t,totgib);
ALPHA_=zeros(totgib,1);
MU_MAT_=zeros(t,2,totgib);
COVAR_=zeros(t,3,totgib);
SM_=zeros(t,totgib);

index_bad=[];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    Gibbs sampling
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
for gib=1:totgib+lgib 
    sm=ones(t,1);    % Classification
    s0=1;
    selv=[1;2];
    selv1=[1;2];
    ppp1_=0;
    
    %generate the prior covariate matrix in each iteration
c0=2; 
C0=eye(2);
sigma0=inv(wishrnd(inv(C0),c0));   % 2x2 draws of covariance matrix 

    
%--------------------------------------------------------------------------
%                         Filtered probabilities
%--------------------------------------------------------------------------
for i=1:t
if i==1 %prediction in the first moment t=1
	selv1=[1;2];
	p1=exp(-lambda(1,:)); %probability of the first observation, expression 3.2
	s0=[s0;(1-p1);p1]; %#ok<AGROW>
	ppp1_=p1;
end

%update step
if i>=2	
    LIK0=zeros(i,1);
    for k=1:i	
        mu_mat=compute_groups_by_kmeans(MU_MAT(1:i,:),k);
        covar_mat=[2*ones(i,1),zeros(i,1),2*ones(i,1)];
        LIK0(k,1) =1000*mvnpdf(Y(i,:),mu_mat(end,:),sigma0);
    end
    for j=1:i
    if j==1
    s0(maxb(i-1,:)+selv(j,:),:)=s0(maxb(i-1,:)+selv(j,:),:)*LIK0(j);  %#ok<SAGROW>
    elseif j<i
    s0(maxb(i-1,:)+selv(j-1,:)+1:maxb(i-1,:)+selv(j,:),:)=s0(maxb(i-1,:)+selv(j-1,:)+1:maxb(i-1,:)+selv(j,:),:)*LIK0(j); 
    elseif j==i
	s0(maxb(i,:),:)=s0(maxb(i,:),:)*LIK0(j); %#ok<SAGROW>
    end
    end
end

if i<t && i>1	
s0(maxb(i-1,:)+1:maxb(i,:),:)=s0(maxb(i-1,:)+1:maxb(i,:),:)/sum(s0(maxb(i-1,:)+1:maxb(i,:),:)); %convert forecast to filter probabilities

           %"prediction step";
            selv1=1;
            for j=i:-1:1
                selv1=[selv1;(selv1(rows(selv1),:)+j)]; %#ok<AGROW>
            end
            s0=[s0;zeros(maxb(i+1,:)-maxb(i,:),1)];  %#ok<AGROW> % Prepare s0 for i+1 by adding zeros 
            pp1=exp(-lambda(1,:));
            pp1=pp1*exp(log(lambda(1,:))*(i-1)-slnf(i-1+1,:));  % Numerator of (3.3) in Koop and Potter (2006)
            if 1-ppp1_>10^(-10)	
                p1=pp1./(1-1*ppp1_);  % P(s2=2/s1=1,d1=1)
            else
                p1=1;
            end
            s0(maxb(i,:)+1,:)=s0(maxb(i-1,:)+1,:)*(1-p1); 
            s0(maxb(i,:)+2,:)=s0(maxb(i-1,:)+1,:)*p1; 
            ppp1_=ppp1_+pp1;  % Denominator of (3.3) in Koop and Potter (2006)
            for j=2:i 
                if j<i
                    p1=exp(-lambda(j,:));
                    p1=p1.*exp(log(lambda(j,:)).*(i-seqa(i,-1,i-j+1))-slnf(1:i-j+1,:));
                    if rows(p1)>1
                        cppp=[1;(1-(cumsum(p1(1:rows(p1)-1,:))))];
                        cpppp=selif(cppp,cppp>10^(-10));
                        iiii=rows(cpppp);
                        if iiii==rows(p1)
                            p1=(p1./cpppp);   % P(st+1=m/st1=m-1,dm)
                        elseif iiii<rows(p1)
                            p1=[(p1(1:iiii,:)./cpppp);ones(rows(p1)-iiii,1)];
                        end
                    end
                    s0(maxb(i,:)+selv1(j-1,:)+2:maxb(i,:)+selv1(j,:),:)=s0(maxb(i-1,:)+selv(j-1,:)+1:maxb(i-1,:)+selv(j,:),:).*(1-p1); % moving up within
                    s0(maxb(i,:)+selv1(j,:)+1,:)=s0(maxb(i-1,:)+selv(j-1,:)+1:maxb(i-1,:)+selv(j,:),:)'*(p1);  % moving across a regime over all ways                  
                elseif j==i
                    s0(maxb(i+1,:)-1,:)=s0(maxb(i,:),:)*(1-exp(-lambda(i,:)));
                    s0(maxb(i+1,:),:)=s0(maxb(i,:),:)*(exp(-lambda(i,:)));                    	
                end
            end
end
        selv=selv1;
        %"predictive probs  "  sum(s0(maxb(i,:)+1:maxb(i+1,:),:));
end
    
%--------------------------------------------------------------------------
%                             GENERATE s y sm
%--------------------------------------------------------------------------
    %forward iteration
    %now draw the states
    s=zeros(t,1);
    p1=s0(maxb(t-1,:)+1:maxb(t,:))/sum(s0(maxb(t-1,:)+1:maxb(t,:)));
    pp1=zeros(t,1);
    pp1(1,:)=p1(1,:);
    for i=2:t
        pp1(i,:)=sum(p1(selv(i-1,:)+1:selv(i,:),:));  % This sums the probabilities of all the elements of p1 that refer to the same k
    end
    cpp1=cumsum(pp1);
    sm(t,:)=maxindc(rand(1,1)<=cpp1);
	if sm(t,:)~= 1   % This happens when there is a structural change in the time series
        cpp1=cumsum(p1(selv(sm(t,:)-1,:)+1:selv(sm(t,:),:))/sum(p1(selv(sm(t,:)-1,:)+1:selv(sm(t,:),:)))); 
        s(t,:)=maxindc(rand(1,1)<=cpp1)-1;
        %back loop
        for i=t-1:-1:2 
            if s(i+1,:)==0 
                sm(i,:)=sm(i+1,:)-1;
                if sm(i,:)==1 
                    s(1:i,:)=seqa(0,1,i);
                    break
                end 
                %find duration of previous regime
                selv=1;
                for j=i-1:-1:1 
                    selv=[selv;(selv(rows(selv),:)+j)]; %#ok<AGROW>
                end
                p1=s0(maxb(i-1,:)+selv(sm(i,:)-1,:)+1:maxb(i-1,:)+selv(sm(i,:),:));
                tsx=cumsum(p1/sum(p1));
                pp1=exp(-lambda(sm(i,:),:));
                if i-sm(i,:) ~=  0 
                    pp1=pp1.*(lambda(sm(i,:),:).^(i-seqa(i,-1,i-sm(i,:)+1)))./(factorial(i-seqa(i,-1,i-sm(i,:)+1)));
                    cpppp=[1;(1-(cumsum(pp1(1:rows(pp1)-1,:))))];
                    cpppp=selif(cpppp,cpppp>10^(-10));
                    iiii=rows(cpppp);
                    if iiii==rows(pp1) 
                        pp1=pp1./cpppp;
                    else
                        pp1=[(pp1(1:iiii,:)./cpppp);ones(rows(pp1)-iiii,1)];
                    end
                    p1=pp1.*p1;
                    cpp1=cumsum(p1/nansum(p1));
                    s(i,:)=maxindc(rand(1,1)<=cpp1)-1;
                else
                    s(i,:)=0;
                end
            else %drop down within this regime
                s(i,:)=s(i+1,:)-1;
                sm(i,:)=sm(i+1,:);
            end
        end
    else
        s(1:t,:)=seqa(0,1,t);
    %goto olivia in the Gauss code original;
end 
   %olivia:
    s0=0;
    K=sm(t);
   
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %              Uninfirmative priors                            %
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %Priors for mu's b0, B0
    sm=sm';
    mu1=zeros(2,K);
    for i=1:K
    index=find(sm==i);
    mu1(:,i)=mean(Y(index,:));
    end
    Y=Y';

    b0=zeros(2,K);
    B0=NaN(2,2,K);  % B0 is the 2x2xK priors of the variances
    B0inv=NaN(2,2,K);
    C0=NaN(2,2,K);
    for k=1:K
       B0k=1000000*eye(2);
       B0(:,:,k)=B0k;
       B0inv(:,:,k)=inv(B0k);
       C0(:,:,k)=eye(2);
    end
    
    % Prior variances inv(sigma)~W(c0,C0). We follow Bensmail et al. (1997):
    c0=repmat(2,1,K);   % 1xK priors of c. See toolbook page 58 
    
    ind=repmat(sm,K,1)==repmat((1:K)',1,t);
    Nk=sum(ind,2)';  % This is 1xK vector Nk(s) 
  
    % Sample variances given means p. 191. If iv2~W[c,C] => iv2/mu,S~W[ck(S),Ck(S)]
    % where ck(S)=c+0.5*Nk(S) and Ck(S)=C+0.5*sumk(Yk-muk)(Yk-muk)'           
    c1=c0+Nk;
    C1=NaN(2,2,K);
    sigma1=NaN(2,2,K);
    sigma1inv=NaN(2,2,K);
    for k=1:K  
        b=(Y(:,sm==k)-repmat(mu1(:,k),1,Nk(k)));
        C1(:,:,k)=C0(:,:,k)+b*b';
        sigma1(:,:,k)=inv(wishrnd(inv(C1(:,:,k)),c1(k)));   % 2x2 dranws of covariance matrix 
        sigma1inv(:,:,k)=inv(sigma1(:,:,k));
    end 

    % Sample means given variances p. 178. If mu~N[b,B] => mu/v2,S~N[bk(S),Bk(S)]
    % where inv(Bk(S))=inv(B)+inv(v2)*Nk(S) and bk(S)=Bk(S)*[inv(v2)*Nk(S)*mean(y/S)+inv(B)]*b] 
    B1=NaN(r,r,K);
    B1inv=NaN(r,r,K);
    b1=NaN(r,K);
    for k=1:K
        B1inv(:,:,k)= B0inv(:,:,k)+Nk(k)*sigma1inv(:,:,k); 
        B1(:,:,k)=inv(B1inv(:,:,k));
        b1(:,k)=B1(:,:,k)*(squeeze(B0inv(:,:,k))*b0(:,k)+sigma1inv(:,:,k)*sum(Y(:,sm==k),2));   
    end
    j=1;
    while j<1000  
        for k=1:K  
            mu1(:,k)=b1(:,k)+chol(B1(:,:,k))'*randn(size(B1(:,:,k),1),1);
            while mu1(1,k)>mu1(2,k) %check peaks before troughts
                mu1(:,k)=b1(:,k)+chol(B1(:,:,k))'*randn(size(B1(:,:,k),1),1);
            end
        end
        result=check_RC(mu1);
        if result==0
            break
        else
            j=j+1;
        end
    end
    if j==1000
         %disp('With these data and parameters it is not possible to guarantee the identification of the reference cycle')
         index_bad=[index_bad;gib-lgib]; %#ok<AGROW>
    end
 
    % Sample and update the poisson intensities
    dur=zeros(t,1);  % Regimes' duration
    sm=sm';
    for i=1:t
        if i>1 && sm(i-1,:)~=sm(i,:) 
            dur(sm(i-1,:),:)=s(i-1,:);
        end
    end
    dur(sm(t,:),:)=s(t,:);
    
    for i=1:t 
        if i<sm(t,:)
            alpha_=alpha+dur(i,:);
            ibeta=(beta+1)^(-1);
            lambda(i,:)=gamrnd(alpha_,ibeta,1,1);
        elseif i==sm(t,:) %duration not fully observed            
            alpha_=alpha;
            ibeta=(beta)^(-1);
            if dur(sm(t,:),:)>0 
                lambdan=gamrnd(alpha_,ibeta,1,1);
                mh=(1-sum(exp(seqa(0,1,dur(sm(t,:),:))*log(lambda(i,:))-lnfact(seqa(0,1,dur(sm(t,:),:))))))/(1-sum(exp(seqa(0,1,dur(sm(t,:),:))*log(lambda(i,:))-lnfact(seqa(0,1,dur(sm(t,:),:))))));
                if mh>rand(1,1) 
                    lambda(i,:)=lambdan;
                end
            else
                lambda(i,:)=gamrnd(alpha_,ibeta,1,1);
            end
        elseif i<t
            alpha_=alpha;
            ibeta=(beta)^(-1);
            lambda(i,:)=gamrnd(alpha_,ibeta,1,1);
        else
            alpha_=alpha;
            ibeta=(beta)^(-1);
            lambda(i,:)=gamrnd(alpha_,ibeta,1,1);
        end
    end


for i=1:K
    index=find(sm==i);
    MU_MAT(index,:)=repmat((mu1(:,i))',length(index),1);
end
Y=Y';

    %%%%%%%%%%%%%%%%%%%%%Storage results%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if gib>lgib 
     index_bad=index_bad(index_bad>0);
     
     K_(gib-lgib)=K;
     LAMBDA_(:,gib-lgib)=lambda;    
     SM_(:,gib-lgib)=sm;
     MU_MAT_(:,:,gib-lgib)=MU_MAT;
    end

    disp('gibbs iteration'); gib  %#ok<NOPTS>
end %end for gib loop

     
     K_(index_bad)=[];
     LAMBDA_(:,index_bad)=[];
     SM_(:,index_bad)=[];
     MU_MAT_(:,:,index_bad)=[];
     
     disp('the mode of K is:'); mode(K_)
     disp('the mean of K is:'); mean(K_)

toc
RDOS_KP.K=K_;
RDOS_KP.LAMBDA=LAMBDA_;
RDOS_KP.SM=SM_;
RDOS_KP.MU_MAT=MU_MAT_;
save('RDOS_KP.mat','RDOS_KP')
